function [ S_x, S_y ] = ChangeFinder( X, T, k, r )

mu_x = 0;
mu_y = 0;
C_x = zeros(k, 1);
C_y = zeros(k, 1);
Sigma_x = 0;
Sigma_y = 0;
X_hat = zeros(length(X), 1);
Y_hat = zeros(length(X), 1);

Y = zeros(length(X), 1);

S_x = zeros(length(X), 1);
S_y = zeros(length(X), 1);

for t=max(T,k):length(X)
    
    % Learn p
    [mu_x, C_x, Sigma_x, X_hat, p] = learn(mu_x, C_x, Sigma_x, X_hat, X);
    
    % Scoring
    if p == 0
        S_x(t) = 0;
    else
        S_x(t) = -log10(p);
    end
    
    % Moving averages of scores
    Y(t) = (1 / T) * sum(S_x(t-T+1:t));
    
    % Learn q
    [mu_y, C_y, Sigma_y, Y_hat, q] = learn(mu_y, C_y, Sigma_y, Y_hat, Y);
    
    % Scoring
    if q == 0
        S_y(t) = 0;
    else
        S_y(t) = -log10(q);
    end
end

    function [mu, C, Sigma, Data_hat, p] = learn(mu, C, Sigma, Data_hat, Data)
        mu = (1-r) * mu + r * Data(t);
        w = 0;
        
        for j = 1:k
            C(j) = (1-r) * C(j) + r * (Data(t) - mu) * (Data(t-j+1) - mu);
        end
        
        tmp = zeros(k,k);
        
        for j = 1:k
            for i = 1:k
                tmp(j,i) = C(abs(j-i)+1);
            end
        end
       
        % avoid rank deficient warning
        if any(tmp) | any(C)
            A = C \ tmp;
        else 
            A = C;
        end
        
        % A = aryule(C,k);
        
        for i = 1:k
            Data_hat(t) = Data_hat(t) + A(i) * (Data(t-i+1) - mu);
            w = w + A(i) * (Data(t-i+1) - mu);
        end
        
        Data_hat(t) = Data_hat(t) + mu;
        w = w + mu;
        
        Sigma = (1-r) * Sigma + r * (Data(t) - Data_hat(t))^2;
        
        p = (1 / ((2 * pi)^(k/2) * sqrt(Sigma))) * exp( -1/2 * (Data(t) - w) * Sigma^(-1) * (Data(t) - w));
    end
end

