function [scores, isCP] = OnlineKernel(X, m1, m2, nu, eta, s)

    sigma = s;

    scores = zeros(1, length(X));
    isCP = [];
    
    GRAM = init_gram_matrix(X);

    for t = (m1+1):(length(X)-m2)
        %fprintf('%.2f%%\n',(t-m1)/(length(X)-m1-m2)*100);

        past = X(t-m1:t-1)';
        future = X(t:t+m2-1)';

        [alpha_past, roh_past, gram_past] = trainSVM(t-m1, t-1, nu, m1);
        [alpha_future, roh_future, gram_future] = trainSVM(t, t+m2-1, nu, m2);
        
        % Calculate difference
        alpha1 = alpha_past;
        alpha2 = alpha_future;
        K11 = gram_past;
        K12 = gram_matrix_new(t-m1, t-1, t, t+m2-1);
        K22 = gram_future;
        ct1ct2 = acos(max(-1, min(1, alpha1'*K12*alpha2 / (sqrt(alpha1'*K11*alpha1) * sqrt(alpha2'*K22*alpha2)))));
        ct1pt1 = acos(max(-1, min(1, roh_past / sqrt(alpha1'*K11*alpha1))));
        ct2pt2 = acos(max(-1, min(1, roh_future / sqrt(alpha2'*K22*alpha2))));
        
        D = ct1ct2 / (ct1pt1 + ct2pt2);
        scores(t) = D;
        if D > eta
            isCP = [isCP t];
        end
    end
    
    function [K] = gram_matrix(X1, X2)
        K = zeros(length(X1), length(X2));
        for i = 1:length(X1)
            for j = 1:length(X2)
                K(i,j) = kcd_gaussian(X1(i), X2(j));
            end
        end
    end

    function [K] = gram_matrix_new(row_start, row_end, column_start, column_end)
        K = GRAM(row_start:row_end, column_start:column_end);
    end

    function [K] = init_gram_matrix(X)
        K = zeros(length(X), length(X));
        for i = 1:length(X)
            for j = i:length(X)
                K(i,j) = kcd_gaussian(X(i), X(j));
            end
        end
        
        % Mirror down to fill other half of matrix
        K = K + K' - diag(diag(K));
    end

    function alpha = blow_up(ind, a)
        alpha = zeros(1, length(ind));
        j = 1;
        for i=1:length(ind)
            if ind(i)
                alpha(i) = a(j);
                j = j + 1;
            end
        end
    end

    function [alpha, roh, gram] = trainSVM(from, to, nu, m)
        data = X(from:to)';
        SVMModel = fitcsvm(data,ones(size(data,1),1),'KernelFunction','gaussian','KernelScale',1/(sqrt(2)*sigma),'Nu',nu);
        %,'Solver','L1QP','OutlierFraction',0.025,'Standardize',true);
        alpha = SVMModel.Alpha ./ (nu * m);
        alpha = blow_up(SVMModel.IsSupportVector, alpha);
        alpha = alpha';
        gram = gram_matrix_new(from, to, from, to);
        % Alternative approach: Do the optimization manually. However this
        % is much slower and the results are worse.
        % len = length(data);
        % alpha = quadprog(gram, [], [], [], ones(1, len), 1, zeros(len,1), ones(len,1)./(nu*m), [] , optimoptions('quadprog', 'TolFun', 1e-20));

        alphaInd = alpha < 1/(nu * m) & alpha > 0;
        roh = alpha' * gram;
        roh = roh(alphaInd);
        % TODO(mh): is this correct? What to do when there is no fitting
        % alpha?
        if numel(roh) == 0
            roh = 0;
        else
            roh = roh(1);
        end
    end

    function G = kcd_gaussian(U,V)
        G = exp(-norm(U - V)^2 / (2 * sigma^2));
    end
end
